//#include <windows.h>
//#include <math.h>
//#include <float.h>
#include "../MachineInterface.h"
#include "basic.h"
#include "parameter.h"

#define MAX_TRACKS		0
#define MIN_TRACKS		0
#define MACHINE_NAME	"Amp"
#define SHORT_NAME		"A"
#define VERSION			"0.1"
#define ABOUTTXT MACHINE_NAME" v"VERSION"\n\nbuild: "__DATE__"\n\nMarcin Dabrowski\nhttp://bigyo.untergrund.net\n"

CMachineAttribute const *pAttributes[] = { NULL };
#define numAttributes	0
CMachineParameter const paraSlider1 = { pt_byte, "Slider1LinLog", "Slider1LinLog", 0, 0x80, 0xFF, MPF_STATE, 0x60 };
CMachineParameter const paraSlider2 = { pt_byte, "Slider2Pow", "Slider2Pow", 0, 0x80, 0xFF, MPF_STATE, 0x60 };
CMachineParameter const para0dBposition = { pt_byte, "0dBposition", "0dBposition", 1, 0x7F, 0xFF, MPF_STATE, 0x60 };
CMachineParameter const *pParameters[] = {	&paraSlider1, &paraSlider2, &para0dBposition };
#define numGlobalParameters		3
#define numTrackParameters		0
////////////////////////////////////////////////////////////////////////////////////
CMachineInfo const MacInfo = 
{
	MT_EFFECT,				// Machine type // MT_GENERATOR or MT_EFFECT
	MI_VERSION,				// Machine interface version
	MIF_STEREO_EFFECT,		// Machine flags
	MIN_TRACKS,				// min tracks
	MAX_TRACKS,				// max tracks
	numGlobalParameters,	// numGlobalParameters
	numTrackParameters,		// numTrackParameters
	pParameters,			// pointer to parameter stuff
	numAttributes,			// numAttributes
	pAttributes,			// pointer to attribute stuff
	MACHINE_NAME,			// Full Machine Name
	SHORT_NAME,				// Short name
	"Marcin Dabrowski",		// Author name
	"&About..."				// Right click menu commands
};
/////////////////////////////////////////////////////////////////////////////////////
#pragma pack(1)                        
class CGval
{
	public:
		byte slider1;
		byte slider2;
		byte _0dbposition;
};
#pragma pack()

class mi;
class miex : public CMachineInterfaceEx { public: mi *pmi; };
class mi : public CMachineInterface
{
	public:
            mi();
			virtual ~mi() {}
			virtual void Tick();
            virtual void Init(CMachineDataInput * const pi);
            virtual bool Work(float *psamples, int numsamples, int const mode);
            virtual char const *DescribeValue(int const param, int const value);
			//virtual void Command(int const i);
			miex ex;
			CMachine * pmac;
	private:
			CGval Gval;
	public:
			parameterLinLog	Slider1;
			parameterPow	Slider2;
			
			float gain1, gain2;
};
//////////////////////
mi::mi() 
{
		ex.pmi = this;
		GlobalVals = &Gval;
		TrackVals = NULL;
		AttrVals = NULL;
}
void mi::Init(CMachineDataInput * const pi)
{
		pCB->SetMachineInterfaceEx(&ex);
		pmac =  pCB->GetThisMachine();
		Slider1.init(&paraSlider1, 0.0, DBToAmplitude(10.0), 1.0, paraSlider1.DefValue);  
		Slider2.init(&paraSlider2, 0.0, DBToAmplitude(10.0), 1.0, paraSlider2.DefValue);  
}
void mi::Tick()
{
	if  (Slider1.update(Gval.slider1))
	{
		gain1 = Slider1();
	}
	if  (Slider2.update(Gval.slider2))
	{
		gain2 = Slider2();
	}
	if (Gval._0dbposition != para0dBposition.NoValue)
	{
		Slider1.slope_set(1.0, Gval._0dbposition);
		Slider2.slope_set(1.0, Gval._0dbposition);
		gain1 = Slider1();
		gain2 = Slider2();
		pCB->InvalidateParameterValueDescription(pmac,0);
		pCB->InvalidateParameterValueDescription(pmac,1);
	}
}

bool mi::Work(float *psamples, int numsamples, int const mode)
{
		if (mode==WM_NOIO)
			return false;
		if (mode==WM_READ)		// <thru>
			return true; 
		if (mode==WM_WRITE) 	// 
			return false;

		//mode==WM_READWRITE
		for (int i=0; i<numsamples*2; i++)
		{
			*psamples++ *= gain1*gain2;
		}
		return true;
}
char const *mi::DescribeValue(int const param, int const value)
{
		static char txt[16];
		switch(param)
		{
				case 0:		return Slider1.describe_db(value);
				case 1:		return Slider2.describe_db(value);
				default:	sprintf(txt, "%d", value); return txt;
		}
}
//void mi::Command(int const i)
//{
//		switch (i)
//		{
//				case 0:
//						MessageBox(NULL,ABOUTTXT,MACHINE_NAME,MB_OK|MB_ICONINFORMATION);
//						break;
//				default:
//						break;
//		}
//}
DLL_EXPORTS