// http://bigyo.untergrund.net

#pragma once

#include "../../buzz/machineinterface.h"
#include "basic.h"

class parameterLin  
{
public:
	parameterLin() {};
	~parameterLin()  {};

	virtual void init( CMachineParameter const * p, double min, double max);
	bool update(int slider_value) ;
	bool change(int slider_value) ;
	virtual double real_value(int slider_value) { return ScaleLin(slider_value, param->MinValue, param->MaxValue, real_min, real_max); }
	
	double operator()(int slider_value) { return real_value(slider_value);}
	double operator()(void) { return real_value(slider_value); }
	
	void parameterLin::describe(char * outtxt, int value,  char * unitdescription);
	char const * describe_hz( int const value);
	char const * describe_db( int const value);

	CMachineParameter const *param;

	int slider_value;
	double real_max;
	double real_min;
};
class parameterLog : public parameterLin 
{
	public:	
		virtual double real_value(int slider_v) { return ScaleLog(slider_v, param->MinValue, param->MaxValue, real_min, real_max); }
};

class parameterPow : public parameterLin 
{
	public:	
		double power;

		void slope_set(double real, int slider)
		{
			double x = ( (double)slider - (double)param->MinValue   ) / ((double)param->MaxValue - (double)param->MinValue);
			double slope = (real - real_min)/(real_max - real_min);
			power = log(slope) / log(x);
		}
		void init( CMachineParameter const * p, double min, double max) 
		{
			parameterLin::init(p,min,max);
			power = 1.0;
		}

		void init( CMachineParameter const * p, double min, double max, double real_v, int slider_v) 
		{
			parameterLin::init(p,min,max);
			slope_set(real_v, slider_v);
		}
		virtual double real_value(int slider_v)
		{
			return ScalePow(slider_v, param->MinValue, param->MaxValue, real_min, real_max, power); 
		}
};

class parameterLinLog : public parameterLin 
{
	public:	
		double slope;

		void slope_set(double real, int slider)
		{
			double y = (real - real_min)/(real_max - real_min);
			double x = ( (double)slider - (double)param->MinValue   ) / ((double)param->MaxValue - (double)param->MinValue);
			if (real_max<real_min)
				y = -y;

			double	real_lin = real_min + x * (real_max-real_min);
			if (real <= real_lin)
				slope = 0.5 * pow ( y / x , 0.5 / (1.0-x) );  // slope [0 .. 0.5]
			else
				slope = 1.0 - 0.5 * pow( (1.0-y)/(1.0-x), 0.5/x ) ;
		}
		void init( CMachineParameter const * p, double min, double max) 
		{
			parameterLin::init(p,min,max);
			slope = 0.5;
		}

		void init( CMachineParameter const * p, double min, double max, double real_v, int slider_v) 
		{
			parameterLin::init(p,min,max);
			slope_set(real_v, slider_v);
		}
		virtual double real_value(int slider_v) 
		{
			return ScaleLinLog( slider_v, param->MinValue, param->MaxValue, real_min, real_max, slope); 
		}
};

///////////////////////////////////////////////////////////////////////////////
void parameterLin::init( CMachineParameter const * p, double min, double max)
{
	param = p;
	real_min  = min;
	real_max  = max;
	slider_value = param->NoValue;
}
bool parameterLin::update(int new_slider_value)
{
	if (new_slider_value != param->NoValue)
	{
		slider_value = new_slider_value;
		return true;
	}
	else return false;
}
bool parameterLin::change(int new_slider_value)
{
	if ( (new_slider_value != param->NoValue) && (new_slider_value != slider_value) )
	{
		slider_value = new_slider_value;
		return true;
	}
	else return false;
}
void parameterLin::describe(char * outtxt, int value,  char * unitdescription)
{
	double fv1 = real_value(value+1);
	double fv  = real_value(value)  ;
	int n  = - log10(fv1-fv);
	if (n<0) n=0;
	sprintf	(outtxt,"%.*f %s", n+1, fv, unitdescription);
}
char const * parameterLin::describe_hz(int const value)
{
	static char txt[16];
	double fv1 = real_value(value+1);
	double fv  = real_value(value)  ;
	int n  = - log10(fv1-fv);
	if (fv>1000.0f)	{ sprintf (txt,"%.*f kHz", (n<-4) ? 0 : n+4 , fv*0.001f) ; }
	else			{ sprintf (txt,"%.*f Hz", (n<-1) ? 0 : n+1 , fv) ; }
	return txt;
}
char const * parameterLin::describe_db(int const value)
{
	static char txt[16];
	float fv  = real_value(value)  ;
	if (fv == 0.0f) return "-Inf. dB";
	else
	{
		fv  = 20.0*log10(fv);
		float fv1 = 20.0*log10(real_value(value+1));
		int n  = - log10(fv1-fv);
		sprintf (txt,"%.*f dB", (n<-1) ? 0 : n+1 , fv) ; 
	}
	return txt;
}
///////////////////////////////////////////////////////////////////////////